"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var inflection_1 = require("inflection");
var env = require("./env");
function swaggerToDataSource(swagger) {
    var scheme = swagger.schemes ? swagger.schemes[0] : 'http';
    var basePath = swagger.host ? scheme + "://" + swagger.host + swagger.basePath : swagger.basePath;
    var auth = {
        type: 'none',
        clientId: null,
        url: null,
        scopes: []
    };
    if (swagger.securityDefinitions) {
        var name_1 = Object.keys(swagger.securityDefinitions)[0];
        var security = swagger.securityDefinitions[name_1];
        auth.scopes = Object.keys(security.scopes).map(function (name) { return ({ name: name }); });
        auth.type = 'oauth';
        if (/client_id=([^&]*)/.test(security.authorizationUrl)) {
            auth.clientId = RegExp.$1;
        }
        auth.url = security.authorizationUrl.replace(/\?client_id.*/, '');
    }
    function parseProperties(definition) {
        return lodash_1.map(definition.properties, function (property, name) {
            var result = Object.assign({ name: name }, property);
            if (property.$ref) {
                result.type = property.$ref.replace('#/definitions/', '');
            }
            else if (property.type == 'array') {
                result.array = true;
                result.type = property.items.$ref.replace('#/definitions/', '');
            }
            else if (property.properties) {
                result.properties = parseProperties(property);
            }
            result.key = !!property['x-key'];
            result.navigation = property['x-navigation'];
            result.foreignKey = property['x-foreignKey'];
            return result;
        });
    }
    function parseSchema(definition, name) {
        return {
            name: name,
            table: definition['x-table'],
            entitySet: inflection_1.pluralize(name),
            properties: parseProperties(definition)
        };
    }
    var schemas = lodash_1.map(swagger.definitions, parseSchema);
    function responseSchema(path, schema) {
        var ref = null;
        if (!schema) {
            return null;
        }
        if (schema.type == 'array') {
            ref = schema.items.$ref;
        }
        else if (schema.$ref) {
            ref = schema.$ref;
        }
        var name = null;
        if (ref) {
            name = ref.replace('#/definitions/', '');
        }
        return name;
    }
    var resources = lodash_1.map(swagger.paths, function (path, name) {
        var url = name.replace(/{([^}]*)}/g, function ($0, $1) {
            return '${' + ("" + env.variable($1)) + '}';
        });
        return {
            path: url,
            operations: lodash_1.map(path, function (operation, method) {
                return {
                    method: method,
                    name: operation.operationId,
                    parameters: lodash_1.map(operation.parameters, function (parameter) {
                        var type = parameter.schema ? responseSchema(name, parameter.schema) : parameter.type;
                        var parameterName = parameter.name;
                        if (!parameterName) {
                            if (parameter.in == 'body') {
                                parameterName = env.variable(env.unwrapType(type));
                            }
                            else {
                                throw new Error('Parameter name not specified.');
                            }
                        }
                        return {
                            in: parameter.in,
                            name: parameterName,
                            required: parameter.required,
                            type: type
                        };
                    }),
                    responses: lodash_1.map(operation.responses, function (response, status) {
                        return {
                            status: status,
                            schema: response.schema
                        };
                    })
                };
            })
        };
    });
    return {
        name: swagger.info ? swagger.info.title : '',
        authorization: auth.type != 'none' ? auth : null,
        connector: swagger.info['x-connector'],
        type: swagger.info['x-type'],
        basePath: basePath,
        resources: resources,
        schemas: schemas
    };
}
exports.swaggerToDataSource = swaggerToDataSource;
//# sourceMappingURL=DataSource.js.map